@@ -1,7 +1,7 @@
 use super::{Command, Stack};
 use crate::{
-    ast::Block, AliasId, BlockId, DeclId, Example, Overlay, OverlayId, ShellError, Signature, Span,
-    Type, VarId, Variable,
+    ast::Block, AliasId, BlockId, Config, DeclId, Example, Overlay, OverlayId, ShellError,
+    Signature, Span, Type, VarId, Variable,
 };
 use core::panic;
 use std::{
@@ -176,14 +176,14 @@ pub struct EngineState {
     pub scope: Vec<ScopeFrame>,
     pub ctrlc: Option<Arc<AtomicBool>>,
     pub env_vars: HashMap<String, Value>,
+    pub config: Config,
     #[cfg(feature = "plugin")]
     pub plugin_signatures: Option<PathBuf>,
 }
 
 pub const NU_VARIABLE_ID: usize = 0;
 pub const IN_VARIABLE_ID: usize = 1;
-pub const CONFIG_VARIABLE_ID: usize = 2;
-pub const ENV_VARIABLE_ID: usize = 3;
+pub const ENV_VARIABLE_ID: usize = 2;
 // NOTE: If you add more to this list, make sure to update the > checks based on the last in the list
 
 impl EngineState {
@@ -205,6 +205,7 @@ impl EngineState {
             scope: vec![ScopeFrame::new()],
             ctrlc: None,
             env_vars: HashMap::new(),
+            config: Config::default(),
             #[cfg(feature = "plugin")]
             plugin_signatures: None,
         }
@@ -263,6 +264,10 @@ impl EngineState {
         if let Some(stack) = stack {
             for mut env_scope in stack.env_vars.drain(..) {
                 for (k, v) in env_scope.drain() {
+                    if k == "config" {
+                        self.config = v.clone().into_config().unwrap_or_default();
+                    }
+
                     self.env_vars.insert(k, v);
                 }
             }
@@ -439,12 +444,15 @@ impl EngineState {
         None
     }
 
-    pub fn find_commands_by_prefix(&self, name: &[u8]) -> Vec<(Vec<u8>, Option<String>)> {
+    pub fn find_commands_by_predicate(
+        &self,
+        predicate: impl Fn(&[u8]) -> bool,
+    ) -> Vec<(Vec<u8>, Option<String>)> {
         let mut output = vec![];
 
         for scope in self.scope.iter().rev() {
             for decl in &scope.decls {
-                if decl.0.starts_with(name) {
+                if predicate(decl.0) {
                     let command = self.get_decl(*decl.1);
                     output.push((decl.0.clone(), Some(command.usage().to_string())));
                 }
@@ -454,12 +462,12 @@ impl EngineState {
         output
     }
 
-    pub fn find_aliases_by_prefix(&self, name: &[u8]) -> Vec<Vec<u8>> {
+    pub fn find_aliases_by_predicate(&self, predicate: impl Fn(&[u8]) -> bool) -> Vec<Vec<u8>> {
         self.scope
             .iter()
             .rev()
             .flat_map(|scope| &scope.aliases)
-            .filter(|decl| decl.0.starts_with(name))
+            .filter(|decl| predicate(decl.0))
             .map(|decl| decl.0.clone())
             .collect()
     }
@@ -474,6 +482,10 @@ impl EngineState {
         panic!("internal error: span missing in file contents cache")
     }
 
+    pub fn get_config(&self) -> &Config {
+        &self.config
+    }
+
     pub fn get_var(&self, var_id: VarId) -> &Variable {
         self.vars
             .get(var_id)
@@ -1233,6 +1245,20 @@ impl<'a> StateWorkingSet<'a> {
         self.permanent_state.env_vars.get(name)
     }
 
+    pub fn get_config(&self) -> &Config {
+        &self.permanent_state.config
+    }
+
+    pub fn list_env(&self) -> Vec<String> {
+        let mut env_vars = vec![];
+
+        for env_var in self.permanent_state.env_vars.clone().into_iter() {
+            env_vars.push(env_var.0)
+        }
+
+        env_vars
+    }
+
     pub fn set_variable_type(&mut self, var_id: VarId, ty: Type) {
         let num_permanent_vars = self.permanent_state.num_vars();
         if var_id < num_permanent_vars {
@@ -1292,34 +1318,40 @@ impl<'a> StateWorkingSet<'a> {
         }
     }
 
-    pub fn find_commands_by_prefix(&self, name: &[u8]) -> Vec<(Vec<u8>, Option<String>)> {
+    pub fn find_commands_by_predicate(
+        &self,
+        predicate: impl Fn(&[u8]) -> bool,
+    ) -> Vec<(Vec<u8>, Option<String>)> {
         let mut output = vec![];
 
         for scope in self.delta.scope.iter().rev() {
             for decl in &scope.decls {
-                if decl.0.starts_with(name) {
+                if predicate(decl.0) {
                     let command = self.get_decl(*decl.1);
                     output.push((decl.0.clone(), Some(command.usage().to_string())));
                 }
             }
         }
 
-        let mut permanent = self.permanent_state.find_commands_by_prefix(name);
+        let mut permanent = self.permanent_state.find_commands_by_predicate(predicate);
 
         output.append(&mut permanent);
 
         output
     }
 
-    pub fn find_aliases_by_prefix(&self, name: &[u8]) -> Vec<Vec<u8>> {
+    pub fn find_aliases_by_predicate(
+        &self,
+        predicate: impl Fn(&[u8]) -> bool + Copy,
+    ) -> Vec<Vec<u8>> {
         self.delta
             .scope
             .iter()
             .rev()
             .flat_map(|scope| &scope.aliases)
-            .filter(|decl| decl.0.starts_with(name))
+            .filter(|decl| predicate(decl.0))
             .map(|decl| decl.0.clone())
-            .chain(self.permanent_state.find_aliases_by_prefix(name))
+            .chain(self.permanent_state.find_aliases_by_predicate(predicate))
             .collect()
     }
 
